package vin.vio.collaboraonline.config.tenant;

import cn.hutool.log.Log;
import cn.hutool.log.LogFactory;
import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.config.listener.Listener;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.yaml.snakeyaml.Yaml;

import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executor;

/**
 * @Description: 获取配置文件中内容，保存为map，供后续调用，支持动态刷新配置文件
 * @Date: 2025/2/26 9:57
 * @Created: by ZZSLL
 */

@Component
public class TenantConfigManager {

    private static final Log log = LogFactory.get();

    // 存储各租户配置，key租户名称
    private final ConcurrentHashMap<String, Map<String, Object>> tenantConfigs = new ConcurrentHashMap<>();

    @Autowired
    private TenantProperties tenantProperties;

    @Value("${spring.cloud.nacos.server-addr}")
    private String nacosServerAddr;

    @Value("${spring.cloud.nacos.config.group}")
    private String group;

    @Value("${spring.cloud.nacos.config.namespace}")
    private String namespace;

    @Value("${spring.cloud.nacos.username}")
    private String username;

    @Value("${spring.cloud.nacos.password}")
    private String password;

    @PostConstruct
    public void init() throws Exception {
        Properties properties = new Properties();
        properties.put("serverAddr", nacosServerAddr);
        properties.put("namespace", namespace);
        properties.put("username", username);
        properties.put("password", password);
        ConfigService configService = NacosFactory.createConfigService(properties);

        for (String tenant : tenantProperties.getList()) {
            String dataId = "application-" + tenant + ".yml";
            String content = configService.getConfig(dataId, group, 5000);
            Map<String, Object> configMap = parseYaml(content);
            tenantConfigs.put(tenant, configMap);

            configService.addListener(dataId, group, new Listener() {
                @Override
                public Executor getExecutor() {
                    return null;
                }

                @Override
                public void receiveConfigInfo(String configInfo) {
                    Map<String, Object> updatedConfig = parseYaml(configInfo);
                    tenantConfigs.put(tenant, updatedConfig);
                    log.info("租户 " + tenant + " 配置已更新！");
                }
            });
        }
    }

    private Map<String, Object> parseYaml(String yamlContent) {
        if (yamlContent == null || yamlContent.trim().isEmpty()) {
            return null;
        }
        Yaml yaml = new Yaml();
        // 将 YAML 转换为 Map，结果结构和 YAML 文件结构一致
        return yaml.load(yamlContent);
    }

    /**
     * 根据租户名称获取对应的配置 Map
     */
    public Map<String, Object> getConfigByTenant(String tenant) {
        return tenantConfigs.get(tenant);
    }
}